################################################
#
#== plot spatial weights for London NO2 example
#== August 2017
#
###############################################

rm(list=ls())

library(akima);
library(fields)
library(MBA)
library(maptools)
library(viridis)

## Set the working directory in R to the reproduce folder before executing the following R-code


path_data="data_files/"
path_fig="figures/"
path_shape="Shape_files/"

##############################################

coords<-read.table(paste(path_data,"coord_w.txt",sep=""),head=T)
coords[which(coords[,2]==max(coords[,2])),"lat"]<-coords[which(coords[,2]==max(coords[,2])),"lat"]+0.05


weights<-read.table(paste(path_data,"weights.txt",sep=""))

weights[,1]<-log(weights[,1]/(1-weights[,1]))

weights[,2]<-(weights[,2]+1)/2
weights[,2]<-log(weights[,2]/(1-weights[,2]))

weights[,3]<-(weights[,3]+1)/2
weights[,3]<-log(weights[,3]/(1-weights[,3]))

weights[,4]<-log(weights[,4]/(1-weights[,4]))

coord.LL<- SpatialPoints(coords, proj4string=CRS("+proj=longlat"))
coord.UTM <- spTransform(coord.LL, CRS("+init=epsg:27700"))


map<-readShapeSpatial(paste(path_shape, "Regions.shp", sep=""))
map<-map[map$NAME=="Greater London Authority",]

#function to extract coordinates from shapefile (by Paul Hiemstra)
allcoordinates_lapply <- function(x) { 
  polys = x@polygons 
  return(do.call("rbind", lapply(polys, function(pp) { 
    do.call("rbind", lapply(pp@Polygons, coordinates)) 
  }))) 
} 
path <- allcoordinates_lapply(map)
dimnames(path)[[2]]<-c("x","y")
path<-as.data.frame(path)
xrange <- range(path$x)
yrange <- range(path$y)

xbox <- xrange + c(-50000, 50000)
ybox <- yrange + c(-50000, 50000)



res=250

xo=seq(495000,570000,res)
yo=seq(150000,201000,res)

surf1<-interp(x=coord.UTM$lon,y=coord.UTM$lat,z=weights[,1],
             xo=xo, yo=yo, linear=FALSE, extrap=TRUE)

surf2<-interp(x=coord.UTM$lon,y=coord.UTM$lat,z=weights[,2],
             xo=xo, yo=yo, linear=FALSE, extrap=TRUE)

surf3<-interp(x=coord.UTM$lon,y=coord.UTM$lat,z=weights[,3],
             xo=xo, yo=yo, linear=FALSE, extrap=TRUE)

surf4<-interp(x=coord.UTM$lon,y=coord.UTM$lat,z=weights[,4],
             xo=xo, yo=yo, linear=FALSE, extrap=TRUE)

surf1$z<-1/(1+exp(-surf1$z))

surf2$z<-1/(1+exp(-surf2$z))
surf2$z<-2*surf2$z-1

surf3$z<-1/(1+exp(-surf3$z))
surf3$z<-2*surf3$z-1

surf4$z<-1/(1+exp(-surf4$z))



########## ===== plot with points ===== ##########

coords<-read.table(paste(path_data,"coord_w.txt",sep=""),head=T)
coord.LL<- SpatialPoints(coords, proj4string=CRS("+proj=longlat"))
coord.UTM <- spTransform(coord.LL, CRS("+init=epsg:27700"))

xlim<-c(490000,575000)
ylim<-c(150000,201000)


cex.axis=1.5
cex.main=2.75
cex.lab=2

dev.new()
par(mfrow=c(2,2), mar=c(4,3,1,1)+0.1) #b,l,t,r

image.plot(surf1, main = paste(""),axes=F,nlevel=64,col = viridis(12),
xlab="",ylab="",cex.lab=cex.lab,
zlim=c(0,1),xlim=xlim,ylim=ylim);
plot(map,add=TRUE,xlab="",ylab="",xlim=xlim,ylim=ylim,main=" ");
polypath(c(path$x, NA, c(xbox, rev(xbox))), c(path$y, NA, rep(ybox, each=2)), col="white", rule="evenodd");
points(x=coord.UTM$lon,y=coord.UTM$lat, pch="+", col="black")
#axis(1,cex.axis=cex.axis, labels=c(NA, 520000,540000, 560000, 580000), at=c(500000,520000,540000,560000,580000));
#axis(2,cex.axis=cex.axis)

image.plot(surf2, main = paste(""),axes=F,nlevel=64,col = viridis(12),
xlab="",ylab="",cex.lab=cex.lab,
zlim=c(-1,1),xlim=xlim,ylim=ylim);
plot(map,add=TRUE,xlab="",ylab="",xlim=xlim,ylim=ylim,main=" ");
polypath(c(path$x, NA, c(xbox, rev(xbox))), c(path$y, NA, rep(ybox, each=2)), col="white", rule="evenodd");
points(x=coord.UTM$lon,y=coord.UTM$lat, pch="+", col="black")
#axis(1,cex.axis=cex.axis, labels=c(NA, 520000,540000, 560000, 580000), at=c(500000,520000,540000,560000,580000));
#axis(2,cex.axis=cex.axis)

image.plot(surf3, main = paste(""),axes=F,nlevel=64,zlim=c(-1,1),col = viridis(12),
xlab="",ylab="",cex.lab=cex.lab,
xlim=xlim,ylim=ylim);
plot(map,add=TRUE,xlab="easting",ylab="northing",xlim=xlim,ylim=ylim,main=" ");
polypath(c(path$x, NA, c(xbox, rev(xbox))), c(path$y, NA, rep(ybox, each=2)), col="white", rule="evenodd");
points(x=coord.UTM$lon,y=coord.UTM$lat,pch="+", col="black")
#axis(1,cex.axis=cex.axis, labels=c(NA, 520000,540000, 560000, 580000), at=c(500000,520000,540000,560000,580000));
#axis(2,cex.axis=cex.axis)

image.plot(surf4, main = paste(""),axes=F,nlevel=64,zlim=c(0,1),col = viridis(12),
xlab="",ylab="",cex.lab=cex.lab,
xlim=xlim,ylim=ylim);
plot(map,add=TRUE,xlab="easting",ylab="northing",xlim=xlim,ylim=ylim,main=" ");
polypath(c(path$x, NA, c(xbox, rev(xbox))), c(path$y, NA, rep(ybox, each=2)), col="white", rule="evenodd");
points(x=coord.UTM$lon,y=coord.UTM$lat, pch="+", col="black")
#axis(1,cex.axis=cex.axis, labels=c(NA, 520000,540000, 560000, 580000), at=c(500000,520000,540000,560000,580000));
#axis(2,cex.axis=cex.axis)


dev.copy2pdf(file=paste(path_fig,"lon_weights_points.pdf",sep=""),width=10,height=7.5)






####################     REPLOT WITH GGPLOT2    #####################
library(ggplot2)
library(tidyr)
library(spBayes)

#fetch coordinates and weights
coords<-read.table(paste(path_data,"coord_w.txt",sep=""),head=T)
coords[which(coords[,2]==max(coords[,2])),"lat"]<-coords[which(coords[,2]==max(coords[,2])),"lat"]+0.05


weights<-read.table(paste(path_data,"weights.txt",sep=""))

#transform weights and coords
weights[,1]<-log(weights[,1]/(1-weights[,1]))

weights[,2]<-(weights[,2]+1)/2
weights[,2]<-log(weights[,2]/(1-weights[,2]))

weights[,3]<-(weights[,3]+1)/2
weights[,3]<-log(weights[,3]/(1-weights[,3]))

weights[,4]<-log(weights[,4]/(1-weights[,4]))

coord.LL<- SpatialPoints(coords, proj4string=CRS("+proj=longlat"))
coord.UTM <- spTransform(coord.LL, CRS("+init=epsg:27700"))

#interpolate
res=250

xo=seq(495000,570000,res)
yo=seq(150000,201000,res)

surf1<-interp(x=coord.UTM$lon,y=coord.UTM$lat,z=weights[,1],
             xo=xo, yo=yo, linear=FALSE, extrap=TRUE)

surf2<-interp(x=coord.UTM$lon,y=coord.UTM$lat,z=weights[,2],
             xo=xo, yo=yo, linear=FALSE, extrap=TRUE)

surf3<-interp(x=coord.UTM$lon,y=coord.UTM$lat,z=weights[,3],
             xo=xo, yo=yo, linear=FALSE, extrap=TRUE)

surf4<-interp(x=coord.UTM$lon,y=coord.UTM$lat,z=weights[,4],
             xo=xo, yo=yo, linear=FALSE, extrap=TRUE)

#back transform interpolations
surf1$z<-1/(1+exp(-surf1$z))

surf2$z<-1/(1+exp(-surf2$z))
surf2$z<-2*surf2$z-1

surf3$z<-1/(1+exp(-surf3$z))
surf3$z<-2*surf3$z-1

surf4$z<-1/(1+exp(-surf4$z))

####

map<-readShapeSpatial(paste(path_shape, "Regions.shp", sep=""))
map<-map[map$NAME=="Greater London Authority",]

#function to extract coordinates from shapefile (by Paul Hiemstra)
allcoordinates_lapply <- function(x) { 
  polys = x@polygons 
  return(do.call("rbind", lapply(polys, function(pp) { 
    do.call("rbind", lapply(pp@Polygons, coordinates)) 
  }))) 
} 


###gives the path for the border
path <- allcoordinates_lapply(map)

grid <- expand.grid(surf1$x,surf1$y)
points<-matrix(c(grid$Var1, grid$Var2),ncol =2, byrow = FALSE)

##which points of thegrid lie inside London
IN<-pointsInPoly(poly = path, points = points)

idx<-1:length(surf1$z)
OUT <- idx[-IN]

surf1$z[OUT]<-NA; surf2$z[OUT]<-NA;
surf3$z[OUT]<-NA; surf4$z[OUT]<-NA

######arrange data in tidy format
interp1<-data.frame(x = surf1$x, surf1$z )
names(interp1)[1:length(surf1$y)+1] <- surf1$y
interp1 <- gather(interp1,key = y,value = weights,-x,convert = TRUE)

interp2<-data.frame(x = surf2$x, surf2$z )
names(interp2)[1:length(surf2$y)+1] <- surf2$y
interp2 <- gather(interp2,key = y,value = weights,-x,convert = TRUE)

interp3<-data.frame(x = surf3$x, surf3$z )
names(interp3)[1:length(surf3$y)+1] <- surf3$y
interp3 <- gather(interp3,key = y,value = weights,-x,convert = TRUE)

interp4<-data.frame(x = surf4$x, surf4$z )
names(interp4)[1:length(surf4$y)+1] <- surf4$y
interp4 <- gather(interp4,key = y,value = weights,-x,convert = TRUE)

#For facetting
interp1$W<-"1"; interp2$W<-"2";
interp3$W<-"3"; interp4$W<-"4"

#Remove rows corresponding to points outside border
interp1<-subset(interp1, subset = !is.na(interp1$weights))
interp2<-subset(interp2, subset = !is.na(interp2$weights))
interp3<-subset(interp3, subset = !is.na(interp3$weights))
interp4<-subset(interp4, subset = !is.na(interp4$weights))


######## set up data frames

df<-data.frame(rbind(interp1,interp2, interp3, interp4))

coord<-data.frame(lon = coord.UTM$lon, lat = coord.UTM$lat )

dimnames(path)[[2]]<-c("x","y")
path<-as.data.frame(path)

########plot all 4 frames


P<-ggplot()+
   geom_raster(data=df, aes(x = x, y = y,fill = weights)) + 
   #geom_point(coord, aes(x=lon,y=lat))+ 
   stat_contour(data=df,aes(x = x, y = y,z = weights), colour = "black", binwidth = 0.1) +
   scale_fill_gradientn(colours = viridis(12), limits = c(-1,1.), guide = "colourbar")+
   facet_wrap(~W, ncol = 2)+
   geom_point(data = coord, aes(x=lon,y=lat))+
   geom_path(data = path, aes(x=x,y=y))
P <-P +  theme(  legend.title=element_text(size=13),
                    legend.text=element_text(size=13),
                    strip.text.x = element_blank(), #strip out facet grid labels
                    strip.text.y = element_blank(),
                    strip.background = element_blank(),
                    axis.line=element_blank(),
                    axis.text.x=element_blank(),
                    axis.text.y=element_blank(),
                    axis.ticks=element_blank(),
                    axis.title.x=element_blank(),
                    axis.title.y=element_blank(),
                 #   legend.position="none",
                 #   panel.background=element_blank(),
                 #   panel.border=element_blank(),
                    panel.grid.major=element_blank(),
                    panel.grid.minor=element_blank(),
                 #   plot.background=element_blank()
                  )
P
dev.copy2pdf(file=paste(path_fig,"lon_weights_pointsGG4.pdf",sep=""),width=10,height=7.5)

########plot all 2 frames

df<-data.frame(rbind(interp1, interp4))

P<-ggplot()+
   geom_raster(data=df, aes(x = x, y = y,fill = weights)) + 
   #geom_point(coord, aes(x=lon,y=lat))+ 
   stat_contour(data=df,aes(x = x, y = y,z = weights), colour = "black", binwidth = 0.1) +
   scale_fill_gradientn(colours = viridis(12), limits = c(0,1.), guide = "colourbar")+
   facet_wrap(~W, ncol = 2)+
   geom_point(data = coord, aes(x=lon,y=lat))+
   geom_path(data = path, aes(x=x,y=y))
P <-P +  theme(  legend.title=element_text(size=13),
                    legend.text=element_text(size=13),
                    strip.text.x = element_blank(), #strip out facet grid labels
                    strip.text.y = element_blank(),
                    strip.background = element_blank(),
                    axis.line=element_blank(),
                    axis.text.x=element_blank(),
                    axis.text.y=element_blank(),
                    axis.ticks=element_blank(),
                    axis.title.x=element_blank(),
                    axis.title.y=element_blank(),
                 #   legend.position="none",
                 #   panel.background=element_blank(),
                 #   panel.border=element_blank(),
                    panel.grid.major=element_blank(),
                    panel.grid.minor=element_blank(),
                 #   plot.background=element_blank()
                  )
P
dev.copy2pdf(file=paste(path_fig,"lon_weights_pointsGG2_main.pdf",sep=""),width=12,height=5)


#######  cross terms


df<-data.frame(rbind(interp2, interp3))

P<-ggplot()+
   geom_raster(data=df, aes(x = x, y = y,fill = weights)) + 
   #geom_point(coord, aes(x=lon,y=lat))+ 
   stat_contour(data=df,aes(x = x, y = y,z = weights), colour = "black", binwidth = 0.1) +
   scale_fill_gradientn(colours = viridis(12), limits = c(-1,1.), guide = "colourbar")+
   facet_wrap(~W, ncol = 2)+
   geom_point(data = coord, aes(x=lon,y=lat))+
   geom_path(data = path, aes(x=x,y=y))
P <-P +  theme(  legend.title=element_text(size=13),
                    legend.text=element_text(size=13),
                    strip.text.x = element_blank(), #strip out facet grid labels
                    strip.text.y = element_blank(),
                    strip.background = element_blank(),
                    axis.line=element_blank(),
                    axis.text.x=element_blank(),
                    axis.text.y=element_blank(),
                    axis.ticks=element_blank(),
                    axis.title.x=element_blank(),
                    axis.title.y=element_blank(),
                 #   legend.position="none",
                 #   panel.background=element_blank(),
                 #   panel.border=element_blank(),
                    panel.grid.major=element_blank(),
                    panel.grid.minor=element_blank(),
                 #   plot.background=element_blank()
                  )
P
dev.copy2pdf(file=paste(path_fig,"lon_weights_pointsGG2_cross.pdf",sep=""),width=12,height=5)


