## Set the working directory in R to the reproduce folder before executing the following R-code
library(maptools)

path_data="data_files/"
path_fig="figures/"
path_shape="Shape_files/"

val.data<-read.table(paste(path_data,"leave_out_coord.txt", sep=""),head=T)
fit.data<-read.table(paste(path_data,"coord.txt", sep=""),head=T)


ldn <- readShapePoly(paste(path_shape,"London_Borough_Excluding_MHW", sep=""))

#change lat lon to 
cord.fit = SpatialPoints(cbind(fit.data$lon, fit.data$lat), proj4string=CRS("+proj=longlat"))
cord.UTM.fit <- spTransform(cord.fit, CRS("+init=epsg:27700"))
cord.val = SpatialPoints(cbind(val.data$lon, val.data$lat), proj4string=CRS("+proj=longlat"))
cord.UTM.val <- spTransform(cord.val, CRS("+init=epsg:27700"))


plot(ldn, 
xlim = range(cord.UTM.fit$coords.x1),
ylim = range(cord.UTM.fit$coords.x2) )
points(cord.UTM.fit,pch="*", col="blue",cex=2.25)
points(cord.UTM.val,pch="+", col="red",cex=2.25)
box(lty="solid")

#dev.copy2pdf(file=paste(path_fig,"LondonNO2_box.pdf", sep=""),width=10,height=12)


##################### recreate in ggplot ########################


library(ggplot2)


cord.UTM.fit<-data.frame(x = cord.UTM.fit$coords.x1, y = cord.UTM.fit$coords.x2)
cord.UTM.fit$site<-"fitting"
cord.UTM.val<-data.frame(x= cord.UTM.val$coords.x1, y = cord.UTM.val$coords.x2)
cord.UTM.val$site<-"validation"

coord<-as.data.frame(rbind(cord.UTM.fit,cord.UTM.val ))


map<-readShapeSpatial(paste(path_shape, "Regions.shp", sep=""))
map<-map[map$NAME=="Greater London Authority",]

#function to extract coordinates from shapefile (by Paul Hiemstra)
allcoordinates_lapply <- function(x) { 
  polys = x@polygons 
  return(do.call("rbind", lapply(polys, function(pp) { 
    do.call("rbind", lapply(pp@Polygons, coordinates)) 
  }))) 
} 


###gives the path for the border

path <- allcoordinates_lapply(map)

dimnames(path)[[2]]<-c("x","y")
path<-as.data.frame(path)

P<-ggplot()+
   geom_point(data = coord, mapping = aes(x=x,y=y,colour = site, shape = site), size = 4  )+
   geom_path(data = path, aes(x=x,y=y))
P <-P +  theme(  legend.title=element_text(size=16),
                    legend.text=element_text(size=16),
                    axis.line=element_blank(),
                    axis.text.x=element_blank(),
                    axis.text.y=element_blank(),
                    axis.ticks=element_blank(),
                    axis.title.x=element_blank(),
                    axis.title.y=element_blank(),
                  #  legend.position="none",
                 #  panel.background=element_blank(),
                 #  panel.border=element_blank(),
                    panel.grid.major=element_blank(),
                    panel.grid.minor=element_blank(),
                  #  plot.background=element_blank()
                  )
P
#dev.copy2pdf(file=paste(path_fig,"LondonNO2_boxGG.pdf", sep=""),width=8,height=6)

